@@ -1,4 +1,4 @@
-// Copyright (C) 1999-2001 Open Source Telecom Corporation.
+// Copyright (C) 2006 Free Software Foundation.
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -59,6 +59,43 @@ namespace ost {
 
 typedef	signed short *Linear;
 
+typedef struct
+{
+	float v2;
+	float v3;
+	float fac;
+} goertzel_state_t;
+
+typedef struct
+{
+	int hit1;
+	int hit2;
+	int hit3;
+	int hit4;
+	int mhit;
+
+	goertzel_state_t row_out[4];
+	goertzel_state_t col_out[4];
+	goertzel_state_t row_out2nd[4];
+	goertzel_state_t col_out2nd[4];
+	goertzel_state_t fax_tone;
+	goertzel_state_t fax_tone2nd;
+	float energy;
+
+	int current_sample;
+	char digits[129];
+	int current_digits;
+	int detected_digits;
+	int lost_digits;
+	int digit_hits[16];
+	int fax_hits;
+} dtmf_detect_state_t;
+
+typedef struct
+{
+	float fac;
+} tone_detection_descriptor_t;
+
 class AudioCodec;
 
 /**
@@ -219,6 +256,8 @@ private:
 	unsigned long length;           // current size of file,
 					// including header
 
+	bool modified;
+
 	void initialize(void);
 	void getWaveFormat(int size);
 
@@ -695,6 +734,33 @@ public:
 		{return next == NULL;};
 };
 
+/**
+ * DTMFDetect is used for detecting DTMF tones in a stream of audio.
+ * It currently only supports 8000Hz input.
+ */
+
+class CCXX_CLASS_EXPORT DTMFDetect
+{
+public:
+	DTMFDetect();
+	~DTMFDetect();
+
+	int putSamples(int16_t buffer[], int count);
+	int getResult(char *buf, int max);
+protected:
+	void goertzelInit(goertzel_state_t *s, tone_detection_descriptor_t *t);
+	void goertzelUpdate(goertzel_state_t *s, int16_t x[], int samples);
+	float goertzelResult(goertzel_state_t *s);
+private:
+	dtmf_detect_state_t *state;
+	tone_detection_descriptor_t dtmf_detect_row[4];
+	tone_detection_descriptor_t dtmf_detect_col[4];
+	tone_detection_descriptor_t dtmf_detect_row_2nd[4];
+	tone_detection_descriptor_t dtmf_detect_col_2nd[4];
+	tone_detection_descriptor_t fax_detect;
+	tone_detection_descriptor_t fax_detect_2nd;
+};
+
 #ifdef	CCXX_NAMESPACES
 };
 #endif
