@@ -1,4 +1,4 @@
-// Copyright (C) 1999-2001 Open Source Telecom Corporation.
+// Copyright (C) 2006 Free Software Foundation.
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -70,6 +70,30 @@ static const char * const ErrorStrs[] = {
 	"errNotPlaying"
 };
 
+AudioCodec *AudioFile::getCodec(void)
+{
+	Encoding e = getEncoding();
+	switch(e)
+	{
+	case alawAudio:
+		return AudioCodec::getCodec(e, "g.711");
+	case mulawAudio:
+		return AudioCodec::getCodec(e, "g.711");
+	case g721ADPCM:
+	case okiADPCM:
+	case voxADPCM:
+		return AudioCodec::getCodec(e, "g.721");
+	case g722_7bit:
+	case g722_6bit:
+		return AudioCodec::getCodec(e, "g.722");
+	case g723_3bit:
+	case g723_5bit:
+		return AudioCodec::getCodec(e, "g.723");
+	default:
+		return NULL;
+	}
+}
+
 void AudioFile::setShort(unsigned char *data, unsigned short val)
 {
 	if(info.order == __BIG_ENDIAN)
@@ -151,6 +175,8 @@ void AudioFile::create(const char *name, Info *myinfo)
 	unsigned char aufile[24];
 	unsigned char riffhdr[40];
 
+	modified = true;
+
 	if(!afCreate(name))
 		return;
 
@@ -326,6 +352,8 @@ void AudioFile::create(const char *name, Info *myinfo)
 		case g723_5bit:
 			setShort(riffhdr + 8, 0x14);
 			setShort(riffhdr + 24, 5);
+		default:
+			break;
 		}
 
 		if(pcm == 0) {
@@ -440,6 +468,8 @@ void AudioFile::create(const char *name, Info *myinfo)
 				strlen(info.annotation) + 1);
 		header = getAbsolutePosition();
 		length = getAbsolutePosition();
+	default:
+		break;
 	}
 }
 
@@ -536,6 +566,8 @@ void AudioFile::open(const char *name)
 	char *ext;
 	unsigned channels;
 
+	modified = false;
+
 	if(!afOpen(name))
 		return;
 
@@ -713,20 +745,33 @@ void AudioFile::open(const char *name)
 
 void AudioFile::close(void)
 {
+	struct stat ino;
 	unsigned char buf[58];
 	if(! isOpen())
 		return;
+
+	if(! modified)
+	{
+		afClose();
+		return;
+	}
+
 	if(! afSeek(0)) {
 		return;
 	}
 
-	if(-1 == afRead(buf, 58)) {
+	if(afRead(buf, 58) < 58) {
+		afClose();
 		return;
 	}
 	afSeek(0);
 
 	switch(info.format) {
 	case riff:
+	case wave:
+		fstat(file.fd, &ino);
+		length = ino.st_size;
+
 		// RIFF header
 		setLong(buf + 4, length - 8);
 
@@ -738,11 +783,14 @@ void AudioFile::close(void)
 			setLong(buf + 40, length - header); 
 
 		if(afWrite(buf, 58) != 58)
+		{
+			afClose();
 			return;
+		}
 
 		break;
-	case sun:
-		// FIXME not implemented
+	default:
+		// FIXME not implemented for sun .au
 		break;
 	}
 	afClose();
@@ -797,6 +845,68 @@ Audio::Error AudioFile::setMinimum(unsigned long samples)
 	return errSuccess;
 }
 
+unsigned AudioFile::getLinear(Linear addr, unsigned samples)
+{
+	int count;
+	AudioCodec *codec;
+
+	if(getEncoding() == pcm16Mono)
+	{
+		count = getBuffer(addr, samples * 2);
+		if(count < 0)
+			return 0;
+		return count / 2;
+	}
+
+	codec = getCodec();
+	if(!codec)
+		return 0;
+
+	count = getCount(info.encoding);
+	samples = (samples / count) * count;
+	count = toBytes(info.encoding, samples);
+	
+	char buffer[count];
+	count = getBuffer(buffer, count);
+	if(count < 1)
+		return 0;
+
+	samples = toSamples(info.encoding, count);
+	return codec->decode(addr, buffer, samples);
+}
+
+unsigned AudioFile::putLinear(Linear addr, unsigned samples)
+{
+	int count;
+	AudioCodec *codec;
+
+	if(getEncoding() == pcm16Mono)
+	{
+		count = putBuffer(addr, samples * 2);
+		if(count < 0)
+			return 0;
+		return count / 2;
+	}
+	codec = getCodec();
+	if(!codec)
+		return 0;
+
+	count = getCount(info.encoding);
+	samples = samples / count * count;
+	count = toBytes(info.encoding, samples);
+	
+	char buffer[count];
+	samples = codec->encode(addr, buffer, samples);
+	if(samples < 1)
+		return 0;
+	count = toBytes(info.encoding, samples);
+	count = putBuffer(buffer, count);
+	if(count < 0)
+		return 0;
+	return toSamples(info.encoding, count);
+}
+
+
 int AudioFile::getBuffer(void *addr, unsigned bytes)
 {
 	char *fname;
@@ -941,6 +1051,28 @@ Audio::Error AudioFile::setLimit(unsigned long samples)
 	return errSuccess;
 }
 
+bool AudioFile::isSigned(void)
+{
+	switch(info.format)
+	{
+	case sun:
+		return false;
+	default:
+		switch(info.encoding)
+		{
+		case pcm8Mono:
+		case pcm8Stereo:
+		case pcm16Mono:
+		case pcm16Stereo:
+		case pcm32Mono:
+		case pcm32Stereo:
+			return true;
+		default:
+			return false;
+		}
+	}
+}
+
 #ifdef	CCXX_NAMESPACES
 };
 #endif
