@@ -1,4 +1,4 @@
-// Copyright (C) 1999-2001 Open Source Telecom Corporation.
+// Copyright (C) 2006 Free Software Foundation.
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@
 
 #include <cc++/config.h>
 #include <cc++/strchar.h>
+#include <cc++/file.h>
 #include <cc++/export.h>
 #include <math.h>
 #include "private.h"
@@ -53,114 +54,65 @@
 namespace ost {
 #endif
 
-AudioRegisterCodec *AudioRegisterCodec::first = NULL;
+Mutex AudioCodec::lock;
+AudioCodec *AudioCodec::first = NULL;
 
-AudioCodec::AudioCodec(unsigned fsize, Encoding encoding, unsigned rate) :
-AudioSample(fsize, encoding, rate)
+AudioCodec::AudioCodec(const char *n, Encoding e)
 {
-	length = fsize;
-}
-
-unsigned AudioCodec::getChannels(void)
-{
-	if(isMono(encoding))
-		return 1;
-	else
-		return 2;
-}
-
-unsigned AudioCodec::load(unsigned char *data, unsigned len)
-{
-	unsigned maxlen = toBytes(encoding, count);
-	if(len > maxlen)
-		len = maxlen;
-
-	memcpy(samples, data, len);
-	length = toSamples(encoding, len);
-	return len;
-}
-
-unsigned AudioCodec::save(unsigned char *data, unsigned maxlen)
-{
-	unsigned len = toBytes(encoding, length);
-	if(len > maxlen)
-		len = maxlen;
-	memcpy(data, samples, len);
-	return len;
-}
-
-AudioRegisterCodec::AudioRegisterCodec(unsigned min, unsigned def)
-{
-	minframe = min;
-	framecnt = def;
+	encoding = e;
+	name = n;
 	next = first;
 	first = this;
 }
 
-AudioCodec *AudioRegisterCodec::findCodec(const char *name, unsigned frames)
+bool AudioCodec::load(const char *name)
 {
-	AudioRegisterCodec *codec = AudioRegisterCodec::first;
+	char path[256];
+	bool success = true;
 
-	while(codec)
-	{
-		if(!stricmp(name, codec->getName()))
-		{
-			if(!frames)
-				frames = codec->getFrameCount();
-			return codec->getCodec(frames);
-		}
-		codec = codec->next;
-	}
-	return NULL;
+	snprintf(path, sizeof(path), "%s/%s", AUDIO_MODULE_PATH, name);             
+	try
+        {
+               new DSO(path);
+        }
+        catch(DSO *dso)
+        {
+               success = false;
+               return false;
+        }
+	return success;
 }
 
-AudioCodec *AudioRegisterCodec::findCodec(Encoding encoding, unsigned rate, unsigned frames)
+AudioCodec *AudioCodec::getCodec(Encoding e, const char *name)
 {
-	AudioRegisterCodec *codec = AudioRegisterCodec::first;
+	bool success;
+	AudioCodec *codec;
+	lock.enterMutex();
+
+	codec = first;
 
 	while(codec)
 	{
-		if(encoding == codec->getEncoding())
-		{
-			if(!rate || codec->getRate() == rate)
-			{
-				if(!frames)
-					frames = codec->getFrameCount();
-				return codec->getCodec(frames);
-			}
-		}
+		if(e == codec->encoding)
+			break;
 		codec = codec->next;
 	}
-	return NULL;
-}
 
-AudioRegisterCodec *AudioRegisterCodec::findRegistration(const char *name)
-{
-	AudioRegisterCodec *codec = AudioRegisterCodec::first;
-
-	while(codec)
+#ifdef	AUDIO_MODULE_PATH
+	if(!codec && name)
 	{
-		if(!stricmp(name, codec->getName()))
-			return codec;
-		codec = codec->next;
+		success = load(name);
+		lock.leaveMutex();
+		if(success)
+			return getCodec(e, NULL);
+		else
+			return NULL;
 	}
-	return NULL;
-}
+#endif
 
-AudioRegisterCodec *AudioRegisterCodec::findRegistration(Encoding encoding, unsigned rate)
-{
-	AudioRegisterCodec *codec = AudioRegisterCodec::first;
+	lock.leaveMutex();
 
-	while(codec)
-	{
-		if(encoding == codec->getEncoding())
-		{
-			if(!rate || codec->getRate() == rate)
-				return codec;
-		}
-		codec = codec->next;
-	}
-	return NULL;
+	return codec;
 }
 
 #ifdef	CCXX_NAMESPACES
