@@ -26,9 +26,6 @@
 //        detriment.
 //
 
-#include <cc++/config.h>
-#include <cc++/strchar.h>
-#include <cc++/export.h>
 #include <math.h>
 #include <stdint.h>
 #include <string.h>
@@ -36,7 +33,6 @@
 #include <stdlib.h>
 #include <time.h>
 #include <fcntl.h>
-#include "private.h"
 #include "audio.h"
 
 /* Basic DTMF specs:
