@@ -1,4 +1,4 @@
-// Copyright (C) 1999-2001 Open Source Telecom Corporation.
+// Copyright (C) 2006 Free Software Foundation.
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@
 
 #include <cc++/config.h>
 #include <cc++/strchar.h>
+#include <cc++/export.h>
 #include <cstdlib>
 #include <cstdio>
 #include "private.h"
@@ -48,6 +49,9 @@
 #ifdef  __WIN32__
 #define FD(x)   ((HANDLE)(x.handle))
 #define SETFD(x,y) (x.handle = (void *)(y))
+#ifndef	INVALID_SET_FILE_POINTER
+#define	INVALID_SET_FILE_POINTER	((DWORD)(-1))
+#endif
 #endif
 
 #ifdef	CCXX_NAMESPACES
@@ -61,7 +65,10 @@ bool AudioFile::afCreate(const char *name)
 	SETFD(file, CreateFile(name, GENERIC_READ | GENERIC_WRITE, 0,
 		NULL, TRUNCATE_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL));
 #else
-	file.fd = ::creat(name, 0660);
+	// JRS: do not use creat() here.  It uses O_RDONLY by default
+	// which prevents us from reading the wave header later on when 
+	// setting the length during AudioFile::Close().
+	file.fd = ::open(name, O_CREAT | O_TRUNC | O_RDWR, 0660);
 #endif
 	return isOpen();
 }
@@ -74,7 +81,7 @@ bool AudioFile::afOpen(const char *name)
 		NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL));
 	if(!isOpen())
 		SETFD(file, CreateFile(name,GENERIC_READ, 0,
-                        NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+                        NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL));
 #else
 	file.fd = ::open(name, O_RDWR);
 	if(file.fd < 0)
@@ -84,8 +91,10 @@ bool AudioFile::afOpen(const char *name)
 }
 int AudioFile::afWrite(unsigned char *data, unsigned len)
 {
+	modified = true;
+
 #ifdef	__WIN32__
-	unsigned int count;
+	DWORD count;
 
 	if(!WriteFile(FD(file), data, (DWORD)len, &count, NULL))
 		return -1;
@@ -98,7 +107,7 @@ int AudioFile::afWrite(unsigned char *data, unsigned len)
 int AudioFile::afRead(unsigned char *data, unsigned len)
 {
 #ifdef	__WIN32__
-	unsigned int count;
+	DWORD count;
 
 	if(!ReadFile(FD(file), data, (DWORD)len, &count, NULL))
 		return -1;
@@ -110,7 +119,7 @@ int AudioFile::afRead(unsigned char *data, unsigned len)
 
 bool AudioFile::afPeek(unsigned char *data, unsigned len)
 {
-	if(afRead(data, len) != len)
+	if(afRead(data, len) != (int)len)
 		return false;
 	return true;
 }
@@ -118,12 +127,13 @@ bool AudioFile::afPeek(unsigned char *data, unsigned len)
 bool AudioFile::afSeek(unsigned long pos)
 {
 #ifdef	__WIN32__
-	SetFilePointer(FD(file), pos, NULL, FILE_BEGIN))
-		return false;
+	if(SetFilePointer(FD(file), pos, NULL, FILE_BEGIN) != INVALID_SET_FILE_POINTER)
 #else
-	::lseek(file.fd, pos, SEEK_SET);
-#endif
-	return true;
+	if(::lseek(file.fd, pos, SEEK_SET) != -1)	
+#endif	
+		return true;
+	else
+		return false;
 }
 
 bool AudioFile::isOpen(void)
@@ -146,10 +156,10 @@ void AudioFile::afClose(void)
 	{
 		size = getPosition();
 		CloseHandle(FD(file));
-		if(size < minimim && pathname)
+		if(size < minimum && pathname)
 			DeleteFile(pathname);
 	}
-	SETFD(file, INVALID_FILE_HANDLE);
+	SETFD(file, INVALID_HANDLE_VALUE);
 #else
 	if(file.fd > -1)
 	{
@@ -188,7 +198,7 @@ Audio::Error AudioFile::setPosition(unsigned long samples)
 #else
         eof = ::lseek(file.fd, 0l, SEEK_END);
 #endif
-        if(samples == ~0l)
+        if(samples == (unsigned long)~0l)
                 return errSuccess;
 
         pos = header + toBytes(info.encoding, samples);
@@ -206,6 +216,25 @@ Audio::Error AudioFile::setPosition(unsigned long samples)
         return errSuccess;
 }
 
+unsigned long AudioFile::getAbsolutePosition(void)
+{
+        unsigned long pos;
+        if(!isOpen())
+                return 0;
+ 
+#ifdef  __WIN32__
+        pos = SetFilePointer(FD(file), 0l, NULL, FILE_CURRENT);
+	if(pos == INVALID_SET_FILE_POINTER) {
+#else
+        pos = ::lseek(file.fd, 0l, SEEK_CUR);
+	if(pos == (unsigned long)-1l) {
+#endif
+		close();
+		return 0;
+	}
+        return pos;
+}
+
 unsigned long AudioFile::getPosition(void)
 {
         unsigned long pos;
@@ -214,9 +243,14 @@ unsigned long AudioFile::getPosition(void)
 
 #ifdef  __WIN32__
         pos = SetFilePointer(FD(file), 0l, NULL, FILE_CURRENT);
+	if(pos == INVALID_SET_FILE_POINTER) {
 #else
-        pos = ::lseek(file.fd, 0l, SEEK_CUR);
+        pos = getAbsolutePosition();
+	if(pos == (unsigned long)-1l) {
 #endif
+		close();
+		return 0;
+	}
         pos = toSamples(info.encoding, pos - header);
         return pos;
 }
